@@ -5,19 +5,26 @@ import re
 import base64
 import requests
 import ssl
+import urllib.parse
+from itertools import zip_longest
 
 from trytond.config import config
+from trytond.i18n import gettext
 from trytond.model import fields
+from trytond.model.exceptions import AccessError
 from trytond.pool import Pool, PoolMeta
 from trytond.transaction import Transaction
 from trytond.wizard import Wizard, StateTransition, StateAction
 
-__all__ = ['PackageType', 'ShipmentOut', 'CreateShipping', 'CreateShippingUPS']
+from trytond.modules.stock_package_shipping.exceptions import (
+    PackingValidationError)
+from .exceptions import UPSError
 
 SERVER_URLS = {
     'testing': 'https://wwwcie.ups.com/json/Ship',
     'production': 'https://onlinetools.ups.com/json/Ship',
     }
+TRACKING_URL = 'https://www.ups.com/track'
 
 
 class PackageType(metaclass=PoolMeta):
@@ -55,43 +62,76 @@ class PackageType(metaclass=PoolMeta):
         return None
 
 
-class ShipmentOut(metaclass=PoolMeta):
-    __name__ = 'stock.shipment.out'
+class Package(metaclass=PoolMeta):
+    __name__ = 'stock.package'
 
-    @classmethod
-    def __setup__(cls):
-        super(ShipmentOut, cls).__setup__()
-        cls._error_messages.update({
-                'warehouse_address_required': ('An address is required for'
-                    ' warehouse "%(warehouse)s".'),
-                'phone_required': ('A phone number is required for'
-                    ' party "%(party)s".'),
-                'shipping_description_required': ('A shipping description is'
-                    ' required for shipment "%(shipment)s".'),
-                })
+    def get_shipping_tracking_url(self, name):
+        pool = Pool()
+        ShipmentOut = pool.get('stock.shipment.out')
+        ShipmentInReturn = pool.get('stock.shipment.in.return')
+        url = super().get_shipping_tracking_url(name)
+        if (self.shipping_reference
+                and self.shipment
+                and self.shipment.id >= 0
+                and self.shipment.carrier
+                and self.shipment.carrier.shipping_service == 'ups'):
+            party = address = None
+            if isinstance(self.shipment, ShipmentOut):
+                party = self.shipment.customer
+                address = self.shipment.delivery_address
+            elif isinstance(self.shipment, ShipmentInReturn):
+                party = self.shipment.supplier
+                address = self.shipment.delivery_address
+            parts = urllib.parse.urlsplit(TRACKING_URL)
+            query = urllib.parse.parse_qsl(parts.query)
+            if party and party.lang and address and address.country:
+                loc = '_'.join(
+                    (party.lang.code.split('_')[0], address.country.code))
+                query.append(('loc', loc))
+            query.append(('tracknum', self.shipping_reference))
+            parts = list(parts)
+            parts[3] = urllib.parse.urlencode(query)
+            url = urllib.parse.urlunsplit(parts)
+        return url
+
+
+class ShippingUPSMixin:
 
     def validate_packing_ups(self):
-        warehouse_address = self.warehouse.address
-        if not warehouse_address:
-            self.raise_user_error('warehouse_address_required', {
-                    'warehouse': self.warehouse.rec_name,
-                    })
-        if warehouse_address.country != self.delivery_address.country:
-            for party in {self.customer, self.company.party}:
+        warehouse = self.shipping_warehouse
+        if not warehouse.address:
+            raise PackingValidationError(
+                gettext('stock_package_shipping_ups'
+                    '.msg_warehouse_address_required',
+                    shipment=self.rec_name,
+                    warehouse=warehouse.rec_name))
+        if warehouse.address.country != self.delivery_address.country:
+            for party in {self.shipping_to, self.company.party}:
                 for mechanism in party.contact_mechanisms:
                     if mechanism.type in ('phone', 'mobile'):
                         break
                 else:
-                    self.raise_user_error('phone_required', {
-                            'party': party.rec_name,
-                            })
+                    raise PackingValidationError(
+                        gettext('stock_package_shipping_ups'
+                            '.msg_phone_required',
+                            shipment=self.rec_name,
+                            party=party.rec_name))
             if not self.shipping_description:
                 if (any(p.type.ups_code != '01' for p in self.root_packages)
                         and self.carrier.ups_service_type != '11'):
                     # TODO Should also test if a country is not in the EU
-                    self.raise_user_error('shipping_description_required', {
-                            'shipment': self.rec_name,
-                            })
+                    raise PackingValidationError(
+                        gettext('stock_package_shipping_ups'
+                            '.msg_shipping_description_required',
+                            shipment=self.rec_name))
+
+
+class ShipmentOut(ShippingUPSMixin, metaclass=PoolMeta):
+    __name__ = 'stock.shipment.out'
+
+
+class ShipmentInReturn(ShippingUPSMixin, metaclass=PoolMeta):
+    __name__ = 'stock.shipment.in.return'
 
 
 class CreateShipping(metaclass=PoolMeta):
@@ -101,12 +141,8 @@ class CreateShipping(metaclass=PoolMeta):
         'stock_package_shipping_ups.act_create_shipping_ups_wizard')
 
     def transition_start(self):
-        pool = Pool()
-        ShipmentOut = pool.get('stock.shipment.out')
-
-        shipment = ShipmentOut(Transaction().context['active_id'])
         next_state = super(CreateShipping, self).transition_start()
-        if shipment.carrier.shipping_service == 'ups':
+        if self.record.carrier.shipping_service == 'ups':
             next_state = 'ups'
         return next_state
 
@@ -125,26 +161,16 @@ class CreateShippingUPS(Wizard):
 
     start = StateTransition()
 
-    @classmethod
-    def __setup__(cls):
-        super(CreateShippingUPS, cls).__setup__()
-        cls._error_messages.update({
-                'ups_webservice_error': ('UPS webservice call failed with the'
-                    ' following error message:\n\n%(message)s'),
-                'has_reference_number': ('Shipment "%(shipment)s" already has'
-                    ' a reference number.'),
-                })
-
     def transition_start(self):
         pool = Pool()
-        ShipmentOut = pool.get('stock.shipment.out')
         Package = pool.get('stock.package')
 
-        shipment = ShipmentOut(Transaction().context['active_id'])
+        shipment = self.record
         if shipment.reference:
-            self.raise_user_error('has_reference_number', {
-                    'shipment': shipment.rec_name,
-                    })
+            raise AccessError(
+                gettext('stock_package_shipping_ups'
+                    '.msg_shipment_has_reference_number',
+                    shipment=shipment.rec_name))
 
         credential = self.get_credential(shipment)
         packages = shipment.root_packages
@@ -167,25 +193,25 @@ class CreateShippingUPS(Wizard):
             error_message = e.args[0]
 
         if error_message:
-            self.raise_user_error('ups_webservice_error', {
-                    'message': error_message,
-                    })
+            raise UPSError(
+                gettext('stock_package_shipping_ups.msg_ups_webservice_error',
+                    message=error_message))
 
         if 'Fault' in response:
             error = response['Fault']['detail']['Errors']
             message = '%s\n\n%s - %s' % (response['Fault']['faultstring'],
                 error['ErrorDetail']['PrimaryErrorCode']['Code'],
                 error['ErrorDetail']['PrimaryErrorCode']['Description'])
-            self.raise_user_error('ups_webservice_error', {
-                    'message': message,
-                    })
+            raise UPSError(
+                gettext('stock_package_shipping_ups.msg_ups_webservice_error',
+                    message=message))
 
         shipment_response = response['ShipmentResponse']
         response_status = shipment_response['Response']['ResponseStatus']
         if response_status['Code'] != '1':
-            self.raise_user_error('ups_webservice_error', {
-                    'message': response_status['Description'],
-                    })
+            raise UPSError(
+                gettext('stock_package_shipping_ups.msg_ups_webservice_error',
+                    message=response_status['Description']))
 
         shipment_results = shipment_response['ShipmentResults']
         shipment.reference = shipment_results['ShipmentIdentificationNumber']
@@ -195,7 +221,7 @@ class CreateShippingUPS(Wizard):
             # instead of a list of one package
             ups_packages = [ups_packages]
 
-        for tryton_pkg, ups_pkg in zip(packages, ups_packages):
+        for tryton_pkg, ups_pkg in zip_longest(packages, ups_packages):
             label = fields.Binary.cast(base64.b64decode(
                     ups_pkg['ShippingLabel']['GraphicImage']))
             tryton_pkg.shipping_reference = ups_pkg['TrackingNumber']
@@ -246,7 +272,7 @@ class CreateShippingUPS(Wizard):
                 'AddressLine': [l[:35]
                     for l in (address.street or '').splitlines()[:3]],
                 'City': address.city[:30],
-                'PostalCode': (address.zip or '').replace(' ', '')[:9],
+                'PostalCode': (address.postal_code or '').replace(' ', '')[:9],
                 'CountryCode': address.country.code if address.country else '',
                 },
             }
@@ -290,19 +316,19 @@ class CreateShippingUPS(Wizard):
                 'Code': package.type.ups_code,
                 },
             }
-        if (package.type.length is not None
-                and package.type.width is not None
-                and package.type.height is not None):
+        if (package.length is not None
+                and package.width is not None
+                and package.height is not None):
             pkg['Dimensions'] = {
                 'UnitOfMeasurement': {
                     'Code': 'CM' if use_metric else 'IN',
                     },
-                'Length': '%i' % round(UoM.compute_qty(package.type.length_uom,
-                        package.type.length, cm if use_metric else inch)),
-                'Width': '%i' % round(UoM.compute_qty(package.type.width_uom,
-                        package.type.width, cm if use_metric else inch)),
-                'Height': '%i' % round(UoM.compute_qty(package.type.height_uom,
-                        package.type.height, cm if use_metric else inch)),
+                'Length': '%i' % round(UoM.compute_qty(package.length_uom,
+                        package.length, cm if use_metric else inch)),
+                'Width': '%i' % round(UoM.compute_qty(package.width_uom,
+                        package.width, cm if use_metric else inch)),
+                'Height': '%i' % round(UoM.compute_qty(package.height_uom,
+                        package.height, cm if use_metric else inch)),
                 }
         if package.total_weight is not None:
             pkg['PackageWeight'] = {
@@ -315,9 +341,8 @@ class CreateShippingUPS(Wizard):
         return pkg
 
     def get_request(self, shipment, packages, credential):
-        warehouse_address = shipment.warehouse.address
-        shipper = self.get_shipping_party(shipment.company.party,
-            warehouse_address)
+        shipper = self.get_shipping_party(
+            shipment.company.party, shipment.shipping_warehouse.address)
         shipper['ShipperNumber'] = credential.account_number
 
         packages = [self.get_package(credential.use_metric, p)
@@ -330,8 +355,8 @@ class CreateShippingUPS(Wizard):
                 'Shipment': {
                     'Description': (shipment.shipping_description or '')[:50],
                     'Shipper': shipper,
-                    'ShipTo': self.get_shipping_party(shipment.customer,
-                        shipment.delivery_address),
+                    'ShipTo': self.get_shipping_party(
+                        shipment.shipping_to, shipment.shipping_to_address),
                     'PaymentInformation': self.get_payment_information(
                         shipment, credential),
                     'Service': {
